@@ -90,8 +90,13 @@
 #include "direct/tracker-direct.h"
 #include "remote/tracker-remote.h"
 
-G_DEFINE_ABSTRACT_TYPE (TrackerSparqlConnection, tracker_sparql_connection,
-                        G_TYPE_OBJECT)
+typedef struct
+{
+	gboolean closing;
+} TrackerSparqlConnectionPrivate;
+
+G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (TrackerSparqlConnection, tracker_sparql_connection,
+                                     G_TYPE_OBJECT)
 
 static void
 tracker_sparql_connection_init (TrackerSparqlConnection *connection)
@@ -140,6 +145,43 @@ tracker_sparql_connection_lookup_dbus_service (TrackerSparqlConnection  *connect
 	                                                                              path);
 }
 
+gboolean
+tracker_sparql_connection_set_error_on_closed (TrackerSparqlConnection  *connection,
+                                               GError                  **error)
+{
+	TrackerSparqlConnectionPrivate *priv =
+		tracker_sparql_connection_get_instance_private (connection);
+
+	if (priv->closing) {
+		g_set_error (error,
+		             G_IO_ERROR,
+		             G_IO_ERROR_CONNECTION_CLOSED,
+		             "Connection is closed");
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+gboolean
+tracker_sparql_connection_report_async_error_on_closed (TrackerSparqlConnection *connection,
+                                                        GAsyncReadyCallback      callback,
+                                                        gpointer                 user_data)
+{
+	TrackerSparqlConnectionPrivate *priv =
+		tracker_sparql_connection_get_instance_private (connection);
+
+	if (priv->closing) {
+		g_task_report_new_error (connection, callback, user_data, NULL,
+		                         G_IO_ERROR,
+		                         G_IO_ERROR_CONNECTION_CLOSED,
+		                         "Connection is closed");
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 /**
  * tracker_sparql_connection_query:
  * @connection: A `TrackerSparqlConnection`
@@ -175,6 +217,9 @@ tracker_sparql_connection_query (TrackerSparqlConnection  *connection,
 	g_return_val_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable), NULL);
 	g_return_val_if_fail (!error || !*error, NULL);
 
+	if (tracker_sparql_connection_set_error_on_closed (connection, error))
+		return NULL;
+
 	cursor = TRACKER_SPARQL_CONNECTION_GET_CLASS (connection)->query (connection,
 	                                                                  sparql,
 	                                                                  cancellable,
@@ -214,6 +259,11 @@ tracker_sparql_connection_query_async (TrackerSparqlConnection *connection,
 	g_return_if_fail (sparql != NULL);
 	g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));
 
+	if (tracker_sparql_connection_report_async_error_on_closed (connection,
+	                                                            callback,
+	                                                            user_data))
+		return;
+
 	TRACKER_SPARQL_CONNECTION_GET_CLASS (connection)->query_async (connection,
 	                                                               sparql,
 	                                                               cancellable,
@@ -287,6 +337,9 @@ tracker_sparql_connection_update (TrackerSparqlConnection  *connection,
 	g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));
 	g_return_if_fail (!error || !*error);
 
+	if (tracker_sparql_connection_set_error_on_closed (connection, error))
+		return;
+
 	TRACKER_SPARQL_CONNECTION_GET_CLASS (connection)->update (connection,
 	                                                          sparql,
 	                                                          cancellable,
@@ -327,6 +380,11 @@ tracker_sparql_connection_update_async (TrackerSparqlConnection *connection,
 	g_return_if_fail (sparql != NULL);
 	g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));
 
+	if (tracker_sparql_connection_report_async_error_on_closed (connection,
+	                                                            callback,
+	                                                            user_data))
+		return;
+
 	TRACKER_SPARQL_CONNECTION_GET_CLASS (connection)->update_async (connection,
 	                                                                sparql,
 	                                                                cancellable,
@@ -389,6 +447,11 @@ tracker_sparql_connection_update_array_async (TrackerSparqlConnection  *connecti
 	g_return_if_fail (sparql != NULL || sparql_length == 0);
 	g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));
 
+	if (tracker_sparql_connection_report_async_error_on_closed (connection,
+	                                                            callback,
+	                                                            user_data))
+		return;
+
 	if (!TRACKER_SPARQL_CONNECTION_GET_CLASS (connection)->update_array_async) {
 		g_task_report_new_error (G_OBJECT (connection), callback, user_data,
 		                         connection,
@@ -481,6 +544,9 @@ tracker_sparql_connection_update_blank (TrackerSparqlConnection  *connection,
 	g_return_val_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable), NULL);
 	g_return_val_if_fail (!error || !*error, NULL);
 
+	if (tracker_sparql_connection_set_error_on_closed (connection, error))
+		return NULL;
+
 	return TRACKER_SPARQL_CONNECTION_GET_CLASS (connection)->update_blank (connection,
 	                                                                       sparql,
 	                                                                       cancellable,
@@ -515,6 +581,11 @@ tracker_sparql_connection_update_blank_async (TrackerSparqlConnection *connectio
 	g_return_if_fail (sparql != NULL);
 	g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));
 
+	if (tracker_sparql_connection_report_async_error_on_closed (connection,
+	                                                            callback,
+	                                                            user_data))
+		return;
+
 	TRACKER_SPARQL_CONNECTION_GET_CLASS (connection)->update_blank_async (connection,
 	                                                                      sparql,
 	                                                                      cancellable,
@@ -586,6 +657,9 @@ tracker_sparql_connection_update_resource (TrackerSparqlConnection  *connection,
 	g_return_val_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable), FALSE);
 	g_return_val_if_fail (!error || !*error, FALSE);
 
+	if (tracker_sparql_connection_set_error_on_closed (connection, error))
+		return FALSE;
+
 	if (!TRACKER_SPARQL_CONNECTION_GET_CLASS (connection)->update_resource) {
 		g_set_error (error,
 		             TRACKER_SPARQL_ERROR,
@@ -632,6 +706,11 @@ tracker_sparql_connection_update_resource_async (TrackerSparqlConnection *connec
 	g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));
 	g_return_if_fail (callback != NULL);
 
+	if (tracker_sparql_connection_report_async_error_on_closed (connection,
+	                                                            callback,
+	                                                            user_data))
+		return;
+
 	if (!TRACKER_SPARQL_CONNECTION_GET_CLASS (connection)->update_resource_async) {
 		g_task_report_new_error (G_OBJECT (connection), callback, user_data,
 		                         connection,
@@ -713,7 +792,7 @@ tracker_sparql_connection_get_namespace_manager (TrackerSparqlConnection *connec
  * This prepared statement can be executed through [method@SparqlStatement.execute]
  * or [method@SparqlStatement.serialize_async] families of functions.
  *
- * Returns: (transfer full) (nullable): A prepared statement
+ * Returns: (transfer full): A prepared statement
  */
 TrackerSparqlStatement *
 tracker_sparql_connection_query_statement (TrackerSparqlConnection  *connection,
@@ -726,6 +805,9 @@ tracker_sparql_connection_query_statement (TrackerSparqlConnection  *connection,
 	g_return_val_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable), NULL);
 	g_return_val_if_fail (!error || !*error, NULL);
 
+	if (tracker_sparql_connection_set_error_on_closed (connection, error))
+		return NULL;
+
 	return TRACKER_SPARQL_CONNECTION_GET_CLASS (connection)->query_statement (connection,
 	                                                                          sparql,
 	                                                                          cancellable,
@@ -744,7 +826,7 @@ tracker_sparql_connection_query_statement (TrackerSparqlConnection  *connection,
  * This prepared statement can be executed through
  * the [method@SparqlStatement.update] family of functions.
  *
- * Returns: (transfer full) (nullable): A prepared statement
+ * Returns: (transfer full): A prepared statement
  *
  * Since: 3.5
  */
@@ -759,6 +841,9 @@ tracker_sparql_connection_update_statement (TrackerSparqlConnection  *connection
 	g_return_val_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable), NULL);
 	g_return_val_if_fail (!error || !*error, NULL);
 
+	if (tracker_sparql_connection_set_error_on_closed (connection, error))
+		return NULL;
+
 	if (!TRACKER_SPARQL_CONNECTION_GET_CLASS (connection)->update_statement) {
 		g_set_error (error,
 		             TRACKER_SPARQL_ERROR,
@@ -812,8 +897,13 @@ tracker_sparql_connection_create_notifier (TrackerSparqlConnection *connection)
 void
 tracker_sparql_connection_close (TrackerSparqlConnection *connection)
 {
+	TrackerSparqlConnectionPrivate *priv =
+		tracker_sparql_connection_get_instance_private (connection);
+
 	g_return_if_fail (TRACKER_IS_SPARQL_CONNECTION (connection));
 
+	priv->closing = TRUE;
+
 	TRACKER_SPARQL_CONNECTION_GET_CLASS (connection)->close (connection);
 }
 
@@ -835,8 +925,13 @@ tracker_sparql_connection_close_async (TrackerSparqlConnection *connection,
                                        GAsyncReadyCallback      callback,
                                        gpointer                 user_data)
 {
+	TrackerSparqlConnectionPrivate *priv =
+		tracker_sparql_connection_get_instance_private (connection);
+
 	g_return_if_fail (TRACKER_IS_SPARQL_CONNECTION (connection));
 
+	priv->closing = TRUE;
+
 	TRACKER_SPARQL_CONNECTION_GET_CLASS (connection)->close_async (connection,
 	                                                               cancellable,
 	                                                               callback,
@@ -899,7 +994,7 @@ tracker_sparql_connection_create_batch (TrackerSparqlConnection *connection)
  * [method@SparqlConnection.query_statement] or [method@SparqlConnection.update_statement]
  * underneath to indistinctly return SPARQL query or update statements.
  *
- * Returns: (transfer full) (nullable): A prepared statement
+ * Returns: (transfer full): A prepared statement
  *
  * Since: 3.3
  **/
@@ -918,6 +1013,9 @@ tracker_sparql_connection_load_statement_from_gresource (TrackerSparqlConnection
 	g_return_val_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable), NULL);
 	g_return_val_if_fail (!error || !*error, NULL);
 
+	if (tracker_sparql_connection_set_error_on_closed (connection, error))
+		return NULL;
+
 	query = g_resources_lookup_data (resource_path,
 	                                 G_RESOURCE_LOOKUP_FLAGS_NONE,
 	                                 error);
@@ -990,6 +1088,11 @@ tracker_sparql_connection_serialize_async (TrackerSparqlConnection *connection,
 	g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));
 	g_return_if_fail (callback != NULL);
 
+	if (tracker_sparql_connection_report_async_error_on_closed (connection,
+	                                                            callback,
+	                                                            user_data))
+		return;
+
 	TRACKER_SPARQL_CONNECTION_GET_CLASS (connection)->serialize_async (connection,
 	                                                                   flags,
 	                                                                   format,
@@ -1069,6 +1172,11 @@ tracker_sparql_connection_deserialize_async (TrackerSparqlConnection *connection
 	g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));
 	g_return_if_fail (callback != NULL);
 
+	if (tracker_sparql_connection_report_async_error_on_closed (connection,
+	                                                            callback,
+	                                                            user_data))
+		return;
+
 	if (!TRACKER_SPARQL_CONNECTION_GET_CLASS (connection)->deserialize_async) {
 		g_task_report_new_error (G_OBJECT (connection), callback, user_data,
 		                         connection,
@@ -1188,7 +1296,7 @@ tracker_sparql_connection_remote_new (const gchar *uri_base)
 /**
  * tracker_sparql_connection_new:
  * @flags: Connection flags to define the SPARQL connection behavior
- * @store: (nullable): The directory that contains the database as a [iface@Gio.File], or %NULL
+ * @store: (nullable): The database location as a [iface@Gio.File], or %NULL
  * @ontology: (nullable): The directory that contains the database schemas as a [iface@Gio.File], or %NULL
  * @cancellable: (nullable): Optional [type@Gio.Cancellable]
  * @error: Error location
@@ -1199,7 +1307,12 @@ tracker_sparql_connection_remote_new (const gchar *uri_base)
  * To connect to databases managed by other processes, use
  * [ctor@SparqlConnection.bus_new].
  *
- * If @store is %NULL, the database will be created in memory.
+ * The @store argument determines where in the local filesystem the database
+ * will be stored. If @store is %NULL, the database will be created in memory.
+ * Traditionally, @store describes a directory where a file named `meta.db`
+ * contains the data. Starting with version 3.10, if the basename of @store
+ * contains a `.` extension separator, it will be considered as the database
+ * file itself.
  *
  * If defined, the @ontology argument must point to a location containing
  * suitable `.ontology` files in Turtle format. These define the structure of
@@ -1221,7 +1334,7 @@ tracker_sparql_connection_remote_new (const gchar *uri_base)
  *
  * Operations on a [class@SparqlConnection] resulting on a
  * `CORRUPT` [error@SparqlError] will have the event recorded
- * persistently through a `.meta.corrupted` file alongside the database files.
+ * persistently through a `.$DB_BASENAME.corrupted` file alongside the database file.
  * If the database is opened without the `READONLY` [flags@SparqlConnectionFlags]
  * flag enabled and the file is found, this constructor will attempt to repair the
  * database. In that situation, this constructor will either return a valid
@@ -1274,7 +1387,7 @@ new_async_cb (GObject      *source,
 /**
  * tracker_sparql_connection_new_async:
  * @flags: Connection flags to define the SPARQL connection behavior
- * @store: (nullable): The directory that contains the database as a [iface@Gio.File], or %NULL
+ * @store: (nullable): The database location as a [iface@Gio.File], or %NULL
  * @ontology: (nullable): The directory that contains the database schemas as a [iface@Gio.File], or %NULL
  * @cancellable: (nullable): Optional [type@Gio.Cancellable]
  * @callback: User-defined [type@Gio.AsyncReadyCallback] to be called when
